@@ -76,7 +76,7 @@ class MailSupport:
         changed = 0
         f = self.folder().aq_base
 
-        # migrate an old zwiki subscribers attribute
+        # migrate an old zwiki subscribers or wikifornow _subscribers attribute
         oldsubs = None
         if (hasattr(f, 'subscribers') and
             type(f.subscribers) is StringType):
@@ -87,23 +87,18 @@ class MailSupport:
             except KeyError:
                 DLOG('failed to delete self.folder().subscribers')
             changed = 1
-
-        # migrate a wikifornow _subscribers attribute
-        oldsubs = None
-        if hasattr(f, '_subscribers'):
+        elif hasattr(f, '_subscribers'):
             oldsubs = f._subscribers.keys()
             try:
                 del f._subscribers
             except KeyError:
                 DLOG('failed to delete self.folder()._subscribers')
             changed = 1
-        if not hasattr(f, 'subscriber_list'):
-            f.subscriber_list = []
-        # copy old subscribers to subscriber_list, unless it's already
-        # got some
-        # XXX merge instead
-        if oldsubs and not f.subscriber_list:
-            f.subscriber_list = oldsubs
+        # ensure a subscriber_list attribute
+        if not hasattr(f, 'subscriber_list'): f.subscriber_list = []
+        # transfer old subscribers to subscriber_list, unless it's already
+        # populated in which case discard them
+        if oldsubs and not f.subscriber_list: f.subscriber_list = oldsubs
 
         # update _properties
         props = map(lambda x:x['id'], f._properties)
@@ -213,6 +208,7 @@ class MailSupport:
                 subs = self._getSubscribers(parent)
                 subs.append(subscriber)
                 self._setSubscribers(subs,parent)
+                if not parent: self.index_object()
         if REQUEST:
             REQUEST.RESPONSE.redirect(
                 REQUEST.get('redirectURL',
@@ -232,6 +228,7 @@ class MailSupport:
                 if string.lower(s) == string.lower(subscriber):
                     sl.remove(s)
             self._setSubscribers(sl,parent)
+            if not parent: self.index_object()
         if REQUEST:
             REQUEST.RESPONSE.redirect(
                 REQUEST.get('redirectURL',
@@ -296,7 +293,7 @@ class MailSupport:
         if self.isWikiSubscriber(subscriber):
             subscriptions.append('whole_wiki')
         # optimization: try to use catalog for memory efficiency..
-        # XXX can we do better - just index subscriber_list and search
+        # XXX can we do better - index subscriber_list and search
         # it directly ?
         if self.hasCatalogIndexesMetadata(
             (['meta_type','path'], ['subscriber_list'])):
@@ -471,7 +468,7 @@ class MailSupport:
 
         # gather bits and pieces
         mhost=self.MailHost
-        username = self.zwiki_username_or_ip(REQUEST)
+        username = self.usernameFrom(REQUEST)
         if not message_id:
             message_id = self.messageIdFromTime(self.ZopeTime())
         mail_from = getattr(self.folder(),'mail_from','')
@@ -497,9 +494,9 @@ class MailSupport:
                 fromhdr = mail_replyto
             else:
                 fromhdr = '%s (%s)' % (mail_replyto,username)
+        # link to message - copied from makeCommentHeading
         pageurl = self.page_url()
-        # link to specific message ? cf makeCommentHeading
-        #pageurl += '#msg'+re.sub(r'^<(.*)>$',r'\1',message_id) 
+        pageurl += '#msg'+re.sub(r'^<(.*)>$',r'\1',message_id) 
         if getattr(self.folder(),'mail_page_name',1):
             pagename = '[%s] ' % self.title_or_id()
         else:
@@ -507,6 +504,7 @@ class MailSupport:
         signature = getattr(self.folder(),
                             'mail_signature',
                             '--\nforwarded from %s' % pageurl)
+                           
         # send message - XXX templatize this
         msg = """\
 From: %s
